@@ -12,21 +12,23 @@
  * Initial revision
  *
  *-------------------------------------------------*/
-
 #include "rain_common.h"
 
 
+
+/* 
+ * sets default values for all packet types
+ */
 void set_defaults(struct pkt_info *pkt,
                   struct pkt_info_tcp *pkttcp,
                   struct pkt_info_icmp *pkticmp,
-#if defined(RAIN_IPV6)
-                  struct pkt_info_icmp6 *pkticmp6,
-#endif
                   struct pkt_info_igmp *pktigmp)
 {
 
-  /* set default pkt info */
-  packetsize      =         576;  /* 576 is the official minimum IP segment size, 1500 for IPv6 */
+  /* 
+   * set default pkt info 
+   */
+  packetsize      =     TCP_MSS;
   verbose         =           0;  /* 0 is the default verbose mode, more output for each nubmer > 0 */
  
   pkt->num        =          -1;  /* infinite */
@@ -42,7 +44,7 @@ void set_defaults(struct pkt_info *pkt,
   }
   pkt->tos        =           0;  /* see rain_inet_common.h */
   pkt->ttl        =          64;  /* well known default */
-  pkt->tot_len    =         0x0;
+  pkt->tot_len    =           0;
   pkt->id         =    getpid();
   pkt->id_of      =           0;
   pkt->frag       =           0;
@@ -52,11 +54,16 @@ void set_defaults(struct pkt_info *pkt,
   pkt->rdport     =           0;
   pkt->rsport     =           0;
   pkt->filler     =        0xA1; /* random hex value (A1 steak sauce!!) */
+  pkt->dfrag      =           0;
   strcpy(pkt->payload,"0");
   strcpy(pkt->gaddr,"127.0.0.1");
+  pkt->sndbuf     =           0;
+  pkt->rcvbuf     =           0;
+  pkt->bcast      =           0;
 
-
-  /* set default pkt_info_tcp */
+  /* 
+   * set default pkt_info_tcp 
+   */
   pkttcp->seq     = getpid();
   pkttcp->window  =       56;
   pkttcp->urg     =        0;
@@ -68,27 +75,24 @@ void set_defaults(struct pkt_info *pkt,
   pkttcp->syn     =        0;
   pkttcp->beenset =        0; /* flag set to 1 when one of the previous flags are set; *
                                * only syn and ack can be combined                      */
+  pkttcp->res     =        0; /* TCP reserved bits */
 
-
-  /* set default pkt_info_icmp */
+  /* 
+   * set default pkt_info_icmp
+   */
   pkticmp->type =    0; /* echo */
   pkticmp->code =    0; /* echo reply */
   pkticmp->id   =    0;
   pkticmp->mtu  = 1500;
   pkticmp->gw   =    0;
 
-#if defined(RAIN_IPV6)
-  /* set default pkt_info_icmp6 */
-  pkticmp6->type = 129;
-  pkticmp6->code =   0; /* echo reply */
-#endif
 
-  /* set default pkt_info_igmp */
+  /* 
+   * set default pkt_info_igmp 
+   */
   pktigmp->type = 0x11; /* membership query */
   pktigmp->code = 0x0; /* max response time: 0 */
 
 
   return;
 }
-
-/* EOF */
