@@ -15,9 +15,15 @@
  * Initial revision
  *
  *-------------------------------------------------*/
-
 #include "rain_common.h"
 
+
+
+/*
+ * siginal_init:
+ * sets up all the signals we'd like
+ * to handle specially
+ */
 extern void 
 signal_init(void) {
   struct sigaction sa_old, sa_new;
@@ -36,33 +42,58 @@ signal_init(void) {
   sigaction(SIGTRAP,&sa_new,&sa_old);
 }
 
-extern void 
+
+/*
+ * Main signal handling
+ *
+ * RETSIGTYPE is a macro defined
+ * by ./configure which will be
+ * set to either int or void, 
+ * depending on the system.
+ */
+extern RETSIGTYPE 
 catch_signal(int signo) {
   time_t total;
   float kbps = 0.0;
   u_long total_size;
 
-   free(buffer);
-   buffer = NULL;
+   freebuf(buffer); /* free our buffered memory  */
 
    switch(signo) {
-    case SIGINT: {
+
+    case SIGINT: { /* Signal Interrupt */
       send_finish = time(NULL);
       total = (send_finish - send_start);
-      if(packets_sent > 100) {
+      if(packets_sent > 49) {
         /* We must do this to avoid a floating point exception */
         total_size = (packetsize*packets_sent);
         kbps  = (float)(total_size/total)/1024;
       }
-      printf("\tThe rain has stopped. %lu drops fell in %ld minutes, %ld seconds (%.1fkBps)\n}\n",
-             packets_sent, (total/60), (total%60), kbps);
+
+
+      if(verbose > 0) {
+        if(total_size < 1024)
+          printf("%lu total bytes sent\n", total_size);
+        else
+          printf("%lu total kilobytes sent\n", (total_size/1024));
+   
+      }
+ 
+
+      printf("%lu drops fell in %d minutes, %d seconds (%.1f kB/s)\n\n",
+             packets_sent, (int)(total/60), (int)(total%60), kbps);
+
+
       exit(0);
     }
+    case SIGALRM: { /* signal alarm (used for timeouts) */
+      fprintf(stderr,"- - Operation timed out\n");
+      fprintf(stderr,"    The remote port may be filtered\n");
+      exit(1);
+    }
     default: {
       fprintf(stderr,"- - %s; Exiting...\n", sys_siglist[signo]);
       exit(1);
     }
   }
 }
-
-/* EOF */
